<?php

namespace Tests\Feature;

use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use App\Models\User;
use Tests\TestCase;
use Hash, Auth;

class UserAuthenticateTest extends TestCase
{
    public function test_authenticate_with_admin_success()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123')
        ]);
        $response = $this->post('/nova/login', [
            'email' => $user->email,
            'password' => '123123123',
            'remember' => false
        ]);
        $user->delete();
        $response->assertStatus(302);
    }

    public function test_authenticate_with_analyst_success()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123')
        ]);
        $response = $this->post('/nova/login', [
            'email' => $user->email,
            'password' => '123123123',
            'remember' => false
        ]);
        $user->delete();
        $response->assertStatus(302);
    }

    public function test_authenticate_with_common_user()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123')
        ]);
        $response = $this->post('/nova/login', [
            'email' => $user->email,
            'password' => '123123123',
            'remember' => false
        ]);
        $user->delete();
        $response->assertStatus(302);
    }

    public function test_authenticate_without_email()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123')
        ]);
        $response = $this->post('/nova/login', [
            'password' => '12312312',
            'remember' => false
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'email' => 'The email field is required.'
        ]);
    }

    public function test_authenticate_without_password()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123')
        ]);
        $response = $this->post('/nova/login', [
            'email' => $user->email,
            'remember' => false
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'password' => 'The password field is required.'
        ]);
    }

    public function test_authenticate_with_non_existing_account()
    {
        $response = $this->post('/nova/login', [
            'email' => 'abc@abc.com',
            'password' => '12345678',
            'remember' => false
        ]);
        $response->assertStatus(302)->assertSessionHasErrors([
            'email' => 'These credentials do not match our records.'
        ]);
    }

    public function test_authenticate_with_wrong_password()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123')
        ]);
        $response = $this->post('/nova/login', [
            'email' => $user->email,
            'password' => '333',
            'remember' => false
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'email' => 'These credentials do not match our records.'
        ]);
    }
}
